#include <libxml/parser.h>
#include <libxml/tree.h>
#include <stdio.h>

static enum {
  PARSER_START,
  PARSER_DISTLIST,
  PARSER_DISTRO,
  PARSER_VENDOR,
  PARSER_VERSION,
  PARSER_CODENAME,
  PARSER_PACKAGE,
  PARSER_FILE,
  PARSER_FINISH
} parserState = PARSER_START;


static void distCharacters(void *state, const xmlChar *chars, int len)
{
  int i;

  switch (parserState) {
  case PARSER_VENDOR:
  case PARSER_CODENAME:
    for (i = 0; i < len; i++)
      putchar(chars[i]);
    break;
  }
}

static void distStartElement(void *state, const xmlChar *name, const xmlChar **attrs)
{
  switch (parserState) {
  case PARSER_START:
    parserState = PARSER_DISTLIST;
    break;
  case PARSER_DISTLIST:
    printf("Distribution:\n");
    while (*attrs) {
      printf("   %s: %s\n", attrs[0], attrs[1]);
      attrs += 2;
    }
    parserState = PARSER_DISTRO;
    break;
  case PARSER_DISTRO:
    if (! strcmp(name, "Vendor")) {
      printf("   Vendor: ");
      parserState = PARSER_VENDOR;
    } else if (! strcmp(name, "Version")) {
      parserState = PARSER_VERSION;
    } else if (! strcmp(name, "CodeName")) {
      printf("   CodeName: ");
      parserState = PARSER_CODENAME;
    } else if (! strcmp(name, "Package")) {
      parserState = PARSER_PACKAGE;
    }
    break;
  case PARSER_PACKAGE:
    if (! strcmp(name, "File")) {
      parserState = PARSER_FILE;
    }
  }
}

static void distEndElement(void *state, const xmlChar *name)
{
  switch (parserState) {
  case PARSER_FILE:
    parserState = PARSER_PACKAGE;
    break;
  case PARSER_VENDOR:
  case PARSER_CODENAME:
    printf("\n");
    /*FALLTHROUGH*/
  case PARSER_VERSION:
  case PARSER_PACKAGE:
    parserState = PARSER_DISTRO;
    break;
  case PARSER_DISTRO:
    parserState = PARSER_DISTLIST;
    break;
  case PARSER_DISTLIST:
    parserState = PARSER_FINISH;
    break;
  }
}

static xmlSAXHandler distListParser = {
    0, /* internalSubset */
    0, /* isStandalone */
    0, /* hasInternalSubset */
    0, /* hasExternalSubset */
    0, /* resolveEntity */
    0, /* getEntity */
    0, /* entityDecl */
    0, /* notationDecl */
    0, /* attributeDecl */
    0, /* elementDecl */
    0, /* unparsedEntityDecl */
    0, /* setDocumentLocator */
    0, /* startDocument */
    0, /* endDocument */
    (startElementSAXFunc) distStartElement, /* startElement */
    (endElementSAXFunc) distEndElement, /* endElement */
    0, /* reference */
    (charactersSAXFunc) distCharacters, /* characters */
    0, /* ignorableWhitespace */
    0, /* processingInstruction */
    0, /* comment */
    0, /* warning */
    0, /* error */
    0, /* fatalError */
};

int main(int argc, char **argv)
{
    xmlSAXParseFile(&distListParser, "distros.xml", 0);

    return 0;
}

